{
  Original Name: Twitter_Common.pas
  Copyright: Copyright (c) 2010, Simon J Stuart, All Rights Reserved
  License: Licensed under the MIT License.
  Author: Simon J Stuart
  Author's Website: http://www.lakraven.com
  Description:
    This unit contains anything which is used by multiple other units
    but doesn't strictly belong to any one of them (mostly Strings).
    This unit will contain a lot more before the end!
  Author's Notes: (See "Twitter.pas")
  Acknowledgements: (See "Twitter.pas")

  Version: 0.1 OFFICIAL OPEN BETA RELEASE
  --------------------------------------------------------------------------------
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  --------------------------------------------------------------------------------
  CODE NOTES: (See "Twitter.pas")

  IMPORTANT NOTICE: You are currently looking at an Open BETA version of TTwitter!
                    This code is subject to a LOT of changes, and whilst you may
                    find portions of this code at present to be messy... rest
                    assured that upon final release, it'll all be nice and clean!
}
unit Twitter_Common;

interface

uses
  Classes, SysUtils;

type
  {
    TTwitter's Custom Exception Types (Indented to represent their descendant levels)
  }
  ETwitterException = class(Exception);
    ETwitterInternalException = class(ETwitterException);
      ETwitterParserException = class(ETwitterInternalException);
      ETwitterHTTPException = class(ETwitterInternalException);
        ETwitterNotLoggedInException = class(ETwitterHTTPException);
        ETwitterXAuthNotAvailableException = class(ETwitterHTTPException);

  {
    Various Enum Types
  }
  TTwitterOAuthParams = (oaRequestToken, oaAccessToken, oaAuthorize, oaAuthenticate);
  TTwitterStatusAction = (saLogin, saLogout, saUserInfo, saTimeline, saPostTweet);
  TTwitterStatusType = (stError, stException);
  TTwitterTimelines = (ttNone, ttFriends, ttHome, ttMentions, ttPublic, ttRetweetedByMe, ttRetweetedToMe, ttRetweetsOfMe, ttUser);
  TTwitterParserType = (ptTimeline, ptRateStatus);

  {
    Various Array Types
  }
  TTwitterStringArray = Array of String;

  {
    TTwitterUserToken holds the user's Token and TokenSecret, along with the Username for the sake of pairing (obviously)
  }
  TTwitterUserToken = record
    Token: String;
    TokenSecret: String;
    Username: String;
    UserID: Int64;
  end;

  {
    Various OnEvent Methods
  }
  TTwitterFunctionBoolean = function(): Boolean of object;
  TTwitterStoreTokenEvent = procedure(const AToken: TTwitterUserToken) of object; // To STORE a Token (Overrides default Registry)
  TTwitterGetTokenEvent = procedure(const AToken: TTwitterUserToken) of object;   // To FETCH a Token (Overrides default Registry)
  TTwitterKillTokenEvent = procedure(const AToken: TTwitterUserToken) of object;  // To DELETE a Token (Overrides default Registry)

  {
    TTwitterApplicationDetails object is used by TTwitter to store the various
    "Application Properties" specific to your application.
    - "Key" and "Secret" are provided by Twitter when you register your application!
    - "CallbackURL" can generally be any VALID URL you like if you're using the integrated
    Twitter Authentication Form included with TTwitter!
    - "ApplicationName" is the name you choose to give to your own Twitter-integrating application!
  }
  TTwitterApplicationDetails = class(TPersistent)
  private
    FApiKey: String;
    FApiSecret: String;
  published
    property ApiKey: String read FApiKey write FApiKey;
    property ApiSecret: String read FApiSecret write FApiSecret;
  end;

  TTwitterProxyDetails = class(TPersistent)
  private
    FBasicAuth: Boolean;
    FPassword: String;
    FPort: Integer;
    FServer: String;
    FUsername: String;
  public
    constructor Create;
  published
    property BasicAuthentication: Boolean read FBasicAuth write FBasicAuth default False;
    property Password: String read FPassword write FPassword;
    property Port: Integer read FPort write FPort default 0;
    property Server: String read FServer write FServer;
    property Username: String read FUsername write FUsername;
  end;

const
  TWITTER_URL_BASE = 'http://api.twitter.com/%d/%s';
  TWITTER_API_VERSION = 1; // ALLOWS US TO USE {$IF} CONDITIONS ON MEMBERS OF CLASSES

resourcestring
  // TTwitter Exceptions
  TTwitter_Except_MissingAPIKey = 'Missing the API Key value.';
  TTwitter_Except_MissingAPISecret = 'Missing the API Secret value.';

  TTwitter_Except_NotLoggedIn = '%s has no logged-in user!';
  TTwitter_Except_ParserFailed = '%s retrieved %s but failed to parse the data!';
  // TTwitter Errors
  TTwitter_Error_StatusLength = 'Twitter Messages must be between 1 and 140 Characters long!';
  // Twitter API - General Error Messages
  TTwitter_Error_304 = 'Not Modified: There was no new data to return!';
  TTwitter_Error_400 = 'Bad Request: The request was invalid (may be due to Rate Limits)';
  TTwitter_Error_401 = 'Unauthorized: Authentication credentials are missing or incorrect!';
  TTwitter_Error_403 = 'Forbidden: The request is understood, but it has been refused (may be due to Update Limits)';
  TTwitter_Error_404 = 'Not Found: The URI requested is invalid or the resource requested, such as a user, does not exist!';
  TTwitter_Error_406 = 'Not Acceptable: invalid format specified in the request!';
  TTwitter_Error_420 = 'Enhance your Calm: You are being rate limited!';
  TTwitter_Error_500 = 'Internal Server Error: Twitter API not available at the moment!';
  TTwitter_Error_502 = 'Bad Gateway: Twitter is down or undergoing maintenance!';
  TTwitter_Error_503 = 'Service Unavailable: Twitter''s servers are too busy!';

implementation

{ TTwitterProxyDetails }

constructor TTwitterProxyDetails.Create;
begin
  Inherited;
  FPort := 0;
  FBasicAuth := False;
end;

end.
